'use strict';

var path = require('path');
var helperModuleImports = require('@babel/helper-module-imports');

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

var NamingRule = {
  LITTLE_CAMEL: 'little-camel',
  BIG_CAMEL: 'big-camel',
  UNDERLINE: 'underline',
  DASH: 'dash',
  FUNCTION: 'FUNCTION'
};

/**
 * @desc 封装了一些项目常用方法.
 */
// 内部函数, 用于判断对象类型
function _getClass(object) {
  return Object.prototype.toString.call(object).match(/^\[object\s(.*)\]$/)[1];
}

function isArray(obj) {
  return _getClass(obj).toLowerCase() === 'array';
}
function isString(obj) {
  return _getClass(obj).toLowerCase() === 'string';
}
function isObject(obj) {
  return _getClass(obj).toLowerCase() === 'object';
}
function isNumber(obj) {
  return _getClass(obj).toLowerCase() === 'number';
}
function isFunction(obj) {
  return _getClass(obj).toLowerCase() === 'function';
}
/**
 * @desc 判断参数是否为空, 包括null, undefined, [], '', {}
 * @param {object} obj 需判断的对象
 */

function isEmpty(obj) {
  var empty = false;

  if (obj === null || obj === undefined) {
    // null and undefined
    empty = true;
  } else if ((isArray(obj) || isString(obj)) && obj.length === 0) {
    empty = true;
  } else if (isObject(obj)) {
    var hasProp = false;

    for (var prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        hasProp = true;
        break;
      }
    }

    if (!hasProp) {
      empty = true;
    }
  } else if (isNumber(obj) && isNaN(obj)) {
    empty = true;
  }

  return empty;
}
/**
 * @desc 判断参数是否为空字符串, 比isEmpty()多判断字符串中全是空格的情况, 如: '   '.
 * @param {string} str 需判断的字符串
 */

function isBlank(str) {
  if (isEmpty(str)) {
    return true;
  } else if (isString(str) && str.trim().length === 0) {
    return true;
  }

  return false;
}
/**
 * @desc 判断参数是否不为空字符串
 */

function isNotBlank(obj) {
  return !isBlank(obj);
}

function toLittleCamel() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/^[A-Z]/, function (match) {
    return match.toLowerCase();
  });
}

function toBigCamel() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/^[a-z]/, function (match) {
    return match.toUpperCase();
  });
}

function toDash() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, function (match, offset) {
    var _str = match.toLowerCase();

    return offset > 0 ? '-' + _str : _str;
  });
}

function toUnderline() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, function (match, offset) {
    var _str = match.toLowerCase();

    return offset > 0 ? '_' + _str : _str;
  });
}

function buildNameByRule(name, rule) {
  switch (rule) {
    case NamingRule.LITTLE_CAMEL:
      // 小驼峰
      name = toLittleCamel(name);
      break;

    case NamingRule.BIG_CAMEL:
      // 大驼峰
      name = toBigCamel(name);
      break;

    case NamingRule.DASH:
      // 中横线
      name = toDash(name);
      break;

    case NamingRule.UNDERLINE:
      // 下划线
      name = toUnderline(name);
      break;

    default:
      if (isString(rule)) {
        // other string
        name = rule.trim();
      } else if (isFunction(rule)) {
        name = rule(name);
      }

  }

  if (!name.startsWith('/')) {
    name = '/' + name;
  }

  return name;
}

function buildPath() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var paths = args.map(function (path$$1) {
    return path$$1 || '';
  });
  return path.join.apply(void 0, _toConsumableArray(paths)).replace(/\\/g, '/');
}

var Plugin =
/*#__PURE__*/
function () {
  function Plugin(_ref) {
    var types = _ref.types;

    _classCallCheck(this, Plugin);

    _defineProperty(this, "_options", void 0);

    _defineProperty(this, "types", void 0);

    _defineProperty(this, "replace", void 0);

    this.types = types;
    this.replace = {};
  }

  _createClass(Plugin, [{
    key: "init",
    value: function init(options) {
      this._options = Object.assign({
        directory: 'lib',
        namingRule: NamingRule.BIG_CAMEL,
        importDefault: true,
        style: false
      }, options);
    }
  }, {
    key: "replaceIdentifierName",
    value: function replaceIdentifierName(path$$1) {
      var identifierName = this.replace[path$$1.node.name];

      if (identifierName && path$$1.scope.hasBinding(identifierName)) {
        path$$1.node.name = identifierName;
      }
    }
  }, {
    key: "ImportSpecifier",
    value: function ImportSpecifier(path$$1, state) {}
  }, {
    key: "ImportDefaultSpecifier",
    value: function ImportDefaultSpecifier(path$$1, state) {// TODO: "foo" in import foo from 'mod'.
    }
  }, {
    key: "ImportNamespaceSpecifier",
    value: function ImportNamespaceSpecifier(path$$1, state) {// TODO: "* as foo" in import * as foo from 'mod'.
    }
  }, {
    key: "ImportDeclaration",
    value: function ImportDeclaration(path$$1, state) {
      var _this = this;

      var source = path$$1.node.source.value;
      var _this$_options = this._options,
          library = _this$_options.library,
          directory = _this$_options.directory,
          namingRule = _this$_options.namingRule,
          importDefault = _this$_options.importDefault,
          style = _this$_options.style;

      if (source === library) {
        path$$1.node.specifiers.forEach(function (specifier) {
          if (_this.types.isImportSpecifier(specifier)) {
            var imported = specifier.imported,
                local = specifier.local; // handle component node
            // var componentDirectory = buildDirectory(library, directory);

            var componentName = buildNameByRule(imported.name, namingRule);
            var componentPath = buildPath(library, directory, componentName); // var componentNode = componentTemplate({
            //     LOCAL_NAME: t.identifier(specifier.local),
            //     SOURCE: t.stringLiteral(componentSource)
            // });

            var newIdentifier;

            if (importDefault) {
              newIdentifier = helperModuleImports.addDefault(path$$1, componentPath);
            } else {
              newIdentifier = helperModuleImports.addNamed(path$$1, local.name, componentPath);
            } // 保存需要替换的标识符名称.


            _this.replace[local.name] = newIdentifier.name; // handle style node

            if (style) {
              // const styleTemplate = template(`
              //     import SOURCE;
              // `);
              var _directory = style.directory,
                  _namingRule = style.namingRule,
                  ext = style.ext;
              var styleDirectory = buildPath(library, directory);
              var styleName = '';
              var extension = '';

              if (_directory) {
                if (_directory.startsWith('/')) {
                  // 从根路径开始设置样式路径, 如: '/less/component'.
                  // styleDirectory = buildDirectory(library, _directory);
                  styleDirectory = buildPath(library, _directory);
                } else {
                  // 在 componentSource 后追加路径, 如: 'style'.
                  styleDirectory = buildPath(componentPath, _directory);
                }
              }

              if (_namingRule) {
                styleName = buildNameByRule(imported.name, _namingRule);
              }

              if (isNotBlank(ext)) {
                extension = ext.trim().startsWith('.') ? ext : '.' + ext;
              } // /antd/lib/button/style(/index.js)        directory = 'style'
              // /antd/lib/button/style/index.less        directory = 'style', namingRule = 'index', extension = 'less'
              // /elemental-ui/less/component/Alert.less  directory = '/less/component' namingRule = 'big-camel', extension = 'less'


              var stylePath = buildPath(styleDirectory, styleName + extension); // let styleNode = styleTemplate({
              //     SOURCE: t.stringLiteral(styleSource)
              // });

              var styleNode = helperModuleImports.addSideEffect(path$$1, stylePath);
            }
          }
        }); // TODO: 将path保存在state的一个队列中, 待退出时集中销毁.

        path$$1.remove();
      }
    }
  }, {
    key: "VariableDeclarator",
    value: function VariableDeclarator(path$$1, state) {}
  }, {
    key: "Identifier",
    value: function Identifier(path$$1, state) {
      this.replaceIdentifierName(path$$1);
    }
  }, {
    key: "MemberExpression",
    value: function MemberExpression(path$$1, state) {}
  }, {
    key: "JSXIdentifier",
    value: function JSXIdentifier(path$$1, state) {
      // 替换JSX中使用到的标识符名称, 确保一致.
      this.replaceIdentifierName(path$$1);
    }
  }, {
    key: "getVisitor",
    value: function getVisitor() {
      var _this2 = this;

      var visitor = {};
      var proto = Object.getPrototypeOf(this);
      Object.getOwnPropertyNames(proto).forEach(function (key) {
        var prop = _this2[key];

        if (key.match(/^[A-Z]/) && typeof prop === 'function') {
          // 注意: 这里把 this 指向了 Plugin 类的实例
          visitor[key] = prop.bind(_this2);
        }
      });
      return visitor;
    }
  }, {
    key: "destroy",
    value: function destroy() {}
  }]);

  return Plugin;
}();

function index (_ref) {
  var types = _ref.types;
  var plugin = new Plugin({
    types: types
  });
  return {
    pre: function pre() {
      plugin.init(this.opts);
    },
    visitor: plugin.getVisitor(),
    post: function post() {
      plugin && plugin.destroy();
    }
  };
}

module.exports = index;
